@@ -4,12 +4,12 @@
 This file is part of WxWidgetsExtensions library.
 -------------------------------------------------------------------------
 
-WxWidgetsExtensions library 0.7.1
+WxExtLib (WxWidgetsExtensions) library
 -----------------------------
 
 COPYRIGHT NOTICE:
 
-WxWidgetsExtensions library Copyright (c) 2003, 2004 Daniel Kps.
+WxExtLib library Copyright (c) 2003-2007 Daniel Kps
 
 The WxWidgetsExtensions library and associated documentation files (the
 "Software") is provided "AS IS".  The author(s) disclaim all
@@ -33,11 +33,19 @@ subject to the following conditions:
  3. This Copyright notice may not be removed or altered from any 
     source or altered source distribution.
 
-End of WxWidgetsExtensions library Copyright notice
+End of WxExtLib library Copyright Notice
 
 -------------------------------------------------------------------------
 */
 
+#ifndef _NO_HEADER_INCLUDE
+
+#include "WxExtLibConfig.h"
+
+#if defined(__GNUG__) && (!defined(__APPLE__)) && (!defined(M_NoPragmaInterface))
+#   pragma implementation "ConfigEntry.cpp"
+#endif
+
 // #include <wx/config.h>
 #include <wx/confbase.h>
 
@@ -46,6 +54,8 @@ End of WxWidgetsExtensions library Copyright notice
 // for possible HACK (see below)
 // #include <wx/fileconf.h>
 
+#endif // _NO_HEADER_INCLUDE
+
 //=========================================================================
 
 wxConfigEntryManager::wxConfigEntryManager()
@@ -82,7 +92,7 @@ wxConfigBase * wxConfigEntryManager::getAssociatedConfig ()
 
 bool wxConfigEntryManager::readAllConfigEntries ()
 {
-    bool IsOk = TRUE;
+    bool IsOk = true;
     // OutputString: intended to collect messages about invalid entries etc.
     // (not yet used)
     wxString OutputString;
@@ -103,7 +113,7 @@ bool wxConfigEntryManager::readAllConfigEntries ()
 
 bool wxConfigEntryManager::writeAllConfigEntries ()
 {
-    bool IsOk = TRUE;
+    bool IsOk = true;
     // OutputString: intended to collect messages about invalid entries etc.
     // (not yet used)
     wxString OutputString; 
@@ -134,7 +144,7 @@ bool wxConfigEntryManager::writeAllConfigEntries ()
 
 bool wxConfigEntryManager::setDefaultValues ()
 {
-    bool IsOk = TRUE;
+    bool IsOk = true;
 
     int ConfigEntryCount = m_ConfigEntryArray.GetCount ();
     for (int Index=0; Index < ConfigEntryCount; ++Index)
@@ -150,6 +160,9 @@ bool wxConfigEntryManager::setDefaultValues ()
 wxConfigEntry::wxConfigEntry ()
 {
     m_IsDontReread = FALSE;
+    m_IsDontWrite = FALSE;
+    m_IsForcedWrite = FALSE;
+
     m_IsAlreadyRead = FALSE;
     m_IsChangePending = FALSE;
     m_IsSetPending = FALSE;
@@ -184,6 +197,16 @@ void wxConfigEntry::setDontReread (bool IsDontReread)
     m_IsDontReread = IsDontReread;
 }
 
+void wxConfigEntry::setDontWrite (bool IsDontWrite)
+{
+    m_IsDontWrite = IsDontWrite;
+}
+
+void wxConfigEntry::setForcedWrite (bool IsForcedWrite)
+{
+    m_IsForcedWrite = IsForcedWrite;
+}
+
 //-------------------------------------------------------------------------
 
 bool wxConfigEntry::readInternal (wxString & String)
@@ -218,6 +241,9 @@ bool wxConfigEntry::readInternal (bool & Boolean)
 
 bool wxConfigEntry::writeInternal (const wxString & String)
 {
+    if (m_IsDontWrite)
+      return true;
+
     if (getIsWriteRequired())
       {
         setWriteState ();
@@ -228,12 +254,15 @@ bool wxConfigEntry::writeInternal (const wxString & String)
       {
         setWriteState ();
         wxASSERT(m_ConfigBase != NULL); // wxConfigEntry possibly not initialized
-        return TRUE;
+        return true;
       }
 }
 
 bool wxConfigEntry::writeInternal (long Long)
 {
+    if (m_IsDontWrite)
+      return true;
+
     if (getIsWriteRequired())
       {
         setWriteState ();
@@ -244,12 +273,15 @@ bool wxConfigEntry::writeInternal (long Long)
       {
         setWriteState ();
         wxASSERT(m_ConfigBase != NULL); // wxConfigEntry possibly not initialized
-        return TRUE;
+        return true;
       }
 }
 
 bool wxConfigEntry::writeInternal (double Double)
 {
+    if (m_IsDontWrite)
+      return true;
+
     if (getIsWriteRequired())
       {
         setWriteState ();
@@ -259,12 +291,15 @@ bool wxConfigEntry::writeInternal (double Double)
     else
       {
         setWriteState ();
-        return TRUE;
+        return true;
       }
 }
 
 bool wxConfigEntry::writeInternal (bool Boolean)
 {
+    if (m_IsDontWrite)
+      return true;
+
     if (getIsWriteRequired())
       {
         setWriteState ();
@@ -274,7 +309,7 @@ bool wxConfigEntry::writeInternal (bool Boolean)
     else
       {
         setWriteState ();
-        return TRUE;
+        return true;
       }
 }
 
@@ -282,7 +317,7 @@ bool wxConfigEntry::writeInternal (bool Boolean)
 
 void wxConfigEntry::setReadState ()
 {
-    m_IsAlreadyRead = TRUE;
+    m_IsAlreadyRead = true;
     m_IsSetPending = FALSE;
     m_IsChangePending = FALSE;
     // m_LastReadTimeMS.Start ();
@@ -302,14 +337,15 @@ bool wxConfigEntry::getIsReadRequired ()
 
 void wxConfigEntry::setWriteState ()
 {
-    m_IsAlreadyRead = TRUE;
+    m_IsAlreadyRead = true;
     m_IsSetPending = FALSE;
     m_IsChangePending = FALSE;
 }
 
 bool wxConfigEntry::getIsWriteRequired ()
 {
-    return m_IsSetPending && m_IsChangePending;
+    return ((m_IsSetPending && m_IsChangePending)
+            || m_IsForcedWrite);
 }
 
 //=========================================================================
@@ -343,6 +379,13 @@ void wxIntegerConfigEntry::get (int & Integer)
     Integer = m_CurrentValue;
 }
 
+int wxIntegerConfigEntry::get ()
+{
+    int Integer;
+    get (Integer);
+    return Integer;
+}
+
 void wxIntegerConfigEntry::get (long & Long)
 {
     int TempInteger;
@@ -366,9 +409,9 @@ void wxIntegerConfigEntry::get (unsigned long & ULong)
 
 void wxIntegerConfigEntry::set (const int & Integer)
 {
-    m_IsSetPending = TRUE;
+    m_IsSetPending = true;
     if ((! m_IsChangePending) && (Integer != m_CurrentValue))
-      m_IsChangePending = TRUE;
+      m_IsChangePending = true;
 
     if (! (m_MinInteger <= Integer && Integer <= m_MaxInteger))
       {
@@ -410,10 +453,10 @@ bool wxIntegerConfigEntry::read (wxString & WXUNUSED(MessageString))
         else
           {
             m_CurrentValue = m_DefaultInteger;
-            m_IsSetPending = TRUE;
-            m_IsChangePending = TRUE;
+            m_IsSetPending = true;
+            m_IsChangePending = true;
           }
-        return TRUE;
+        return true;
       }
     else
       return FALSE;
@@ -452,11 +495,18 @@ void wxBooleanConfigEntry::get (bool & Boolean)
     Boolean = m_CurrentValue;
 }
 
+bool wxBooleanConfigEntry::get ()
+{
+    bool Boolean;
+    get (Boolean);
+    return Boolean;
+}
+
 void wxBooleanConfigEntry::set (const bool & Boolean)
 {
-    m_IsSetPending = TRUE;
+    m_IsSetPending = true;
     if ((! m_IsChangePending) && (Boolean != m_CurrentValue))
-      m_IsChangePending = TRUE;
+      m_IsChangePending = true;
 
     m_CurrentValue = Boolean;
 }
@@ -471,7 +521,7 @@ void wxBooleanConfigEntry::get (BOOL & Boolean)
 
 void wxBooleanConfigEntry::set (const BOOL & Boolean)
 {
-    bool TempBoolean = ((Boolean) ? TRUE : FALSE);
+    bool TempBoolean = ((Boolean) ? true : FALSE);
     set (TempBoolean);
 }
 #endif
@@ -495,10 +545,10 @@ bool wxBooleanConfigEntry::read (wxString & WXUNUSED(MessageString))
         else
           {
             m_CurrentValue = m_DefaultBoolean;
-            m_IsSetPending = TRUE;
-            m_IsChangePending = TRUE;
+            m_IsSetPending = true;
+            m_IsChangePending = true;
           }
-        return TRUE;
+        return true;
       }
     else
       return FALSE;
@@ -537,11 +587,18 @@ void wxStringConfigEntry::get (wxString & String)
     String = m_CurrentValue;
 }
 
+wxString wxStringConfigEntry::get ()
+{
+    wxString String;
+    get (String);
+    return String;
+}
+
 void wxStringConfigEntry::set (const wxString & String)
 {
-    m_IsSetPending = TRUE;
+    m_IsSetPending = true;
     if ((! m_IsChangePending) && (String != m_CurrentValue))
-      m_IsChangePending = TRUE;
+      m_IsChangePending = true;
 
     m_CurrentValue = String;
 }
@@ -565,10 +622,10 @@ bool wxStringConfigEntry::read (wxString & WXUNUSED(MessageString))
         else
           {
             m_CurrentValue = m_DefaultString;
-            m_IsSetPending = TRUE;
-            m_IsChangePending = TRUE;
+            m_IsSetPending = true;
+            m_IsChangePending = true;
           }
-        return TRUE;
+        return true;
       }
     else
       return FALSE;
@@ -580,4 +637,239 @@ void wxStringConfigEntry::setToDefault ()
 }
 
 //=========================================================================
+
+wxDoubleConfigEntry::wxDoubleConfigEntry()
+{
+    setRangeAndDefault (0., 0., 0.);
+    m_CurrentValue = 0.;
+}
+
+void wxDoubleConfigEntry::init (const wxString & EntryPath, double MinDouble,
+                                double MaxDouble, double DefaultDouble)
+{
+    setEntryPath (EntryPath);
+    setRangeAndDefault (MinDouble, MaxDouble, DefaultDouble);
+    m_CurrentValue = 0.;
+}
+
+void wxDoubleConfigEntry::setRangeAndDefault (double MinDouble, double MaxDouble, double DefaultDouble)
+{
+    m_MinDouble = MinDouble;
+    m_MaxDouble = MaxDouble;
+    m_DefaultDouble = DefaultDouble;
+}
+
+void wxDoubleConfigEntry::get (double & Double)
+{
+    wxString MessageString;
+    read (MessageString);
+
+    Double = m_CurrentValue;
+}
+
+double wxDoubleConfigEntry::get ()
+{
+    double Double;
+    get (Double);
+    return Double;
+}
+
+void wxDoubleConfigEntry::set (const double & Double)
+{
+    m_IsSetPending = true;
+    if ((! m_IsChangePending) && (Double != m_CurrentValue))
+      m_IsChangePending = true;
+
+    if (! (m_MinDouble <= Double && Double <= m_MaxDouble))
+      {
+        // NOTE this case could be an indication of an application internal
+        // error
+        m_CurrentValue = m_DefaultDouble;
+      }
+    else
+      {
+        m_CurrentValue = Double;
+      }
+}
+
+//-------------------------------------------------------------------------
+
+bool wxDoubleConfigEntry::write (wxString & WXUNUSED(MessageString))
+{
+    double TempDouble = m_CurrentValue;
+    return writeInternal (TempDouble);
+}
+
+bool wxDoubleConfigEntry::read (wxString & WXUNUSED(MessageString))
+{
+    double Double = m_CurrentValue;
+    if (getIsReadRequired())
+      {
+        if (readInternal (Double))
+          {
+            double TempDouble = Double;
+            if (m_MinDouble <= TempDouble && TempDouble <= m_MaxDouble)
+              {
+                m_CurrentValue = TempDouble;
+              }
+            else
+              {
+                m_CurrentValue = m_DefaultDouble;
+              }
+          }
+        else
+          {
+            m_CurrentValue = m_DefaultDouble;
+            m_IsSetPending = true;
+            m_IsChangePending = true;
+          }
+        return true;
+      }
+    else
+      return FALSE;
+}
+
+void wxDoubleConfigEntry::setToDefault ()
+{
+    set (m_DefaultDouble);
+}
+
+//=========================================================================
+
+wxDateTimeConfigEntry::wxDateTimeConfigEntry()
+{
+    // setRangeAndDefault (0, 0, 0);
+    // m_CurrentValue = 0;
+}
+
+void wxDateTimeConfigEntry::init (const wxString & EntryPath, 
+                                  const wxDateTime & MinDateTime,
+                                  const wxDateTime & MaxDateTime,
+                                  const wxDateTime & DefaultDateTime)
+{
+    setEntryPath (EntryPath);
+    setRangeAndDefault (MinDateTime, MaxDateTime, DefaultDateTime);
+    m_CurrentValue = wxDateTime ();
+}
+
+void wxDateTimeConfigEntry::setRangeAndDefault (const wxDateTime & MinDateTime, 
+                                                const wxDateTime & MaxDateTime, 
+                                                const wxDateTime & DefaultDateTime)
+{
+    m_MinDateTime = MinDateTime;
+    m_MaxDateTime = MaxDateTime;
+    m_DefaultDateTime = DefaultDateTime;
+}
+
+void wxDateTimeConfigEntry::get (wxDateTime & DateTime)
+{
+    wxString MessageString;
+    read (MessageString);
+
+    DateTime = m_CurrentValue;
+}
+
+wxDateTime wxDateTimeConfigEntry::get ()
+{
+    wxDateTime DateTime;
+    get (DateTime);
+    return DateTime;
+}
+
+void wxDateTimeConfigEntry::set (const wxDateTime & DateTime)
+{
+    m_IsSetPending = true;
+    // (!m_CurrentValue.IsValid())
+    if ((! m_IsChangePending) && (DateTime != m_CurrentValue))
+      {
+        m_IsChangePending = true;
+      }
+
+    // if (!((!m_MinDateTime.IsValid()) || (m_MinDateTime <= DateTime) 
+    // && ((!m_MaxDateTime.IsValid()) || (DateTime <= m_MaxDateTime))))
+    if (0)
+      {
+        wxASSERT (0);
+        // NOTE this case could be an indication of an application internal
+        // error
+        m_CurrentValue = m_DefaultDateTime;
+      }
+    else
+      {
+        m_CurrentValue = DateTime;
+      }
+}
+
+//-------------------------------------------------------------------------
+
+bool wxDateTimeConfigEntry::write (wxString & WXUNUSED(MessageString))
+{
+    wxString DateTimeString;
+    if (m_CurrentValue.IsValid())
+      DateTimeString = m_CurrentValue.FormatISODate() << " " << m_CurrentValue.FormatISOTime();
+
+    return writeInternal (DateTimeString);
+}
+
+bool wxDateTimeConfigEntry::read (wxString & WXUNUSED(MessageString))
+{
+    wxDateTime TempDateTime;
+    if (getIsReadRequired())
+      {
+        if (readDateTime (TempDateTime)
+            && TempDateTime.IsValid())
+          {
+            if (((!m_MinDateTime.IsValid()) || (m_MinDateTime <= TempDateTime))
+                && ((!m_MaxDateTime.IsValid()) || (TempDateTime <= m_MaxDateTime)))
+              {
+                m_CurrentValue = TempDateTime;
+              }
+            else
+              {
+                m_CurrentValue = m_DefaultDateTime;
+              }
+          }
+        else
+          {
+            m_CurrentValue = m_DefaultDateTime;
+            m_IsSetPending = true;
+            m_IsChangePending = true;
+          }
+        return true;
+      }
+    else
+      return FALSE;
+}
+
+bool wxDateTimeConfigEntry::readDateTime(wxDateTime & DateTime)
+{
+    wxString ISODateTimeFormatString = "%Y-%m-%d %H:%M:%S";
+
+    wxDateTime TempDateTime;
+    wxString TempDateTimeString;
+
+    if (!readInternal (TempDateTimeString))
+      return false;
+
+    // TODO handling of parse results could be improved, this should be put into
+    // a new parseDateTime() function (similar to existing parseDate())
+    // TODO add support for date-only and time-only entries
+    // TODO use more strict function parseDateTimeWithFormat()
+    if (TempDateTime.ParseFormat (TempDateTimeString, ISODateTimeFormatString) != NULL)
+      {
+        DateTime = TempDateTime;
+        return true;
+      }
+    else
+      {
+        return false;
+      }
+}
+
+void wxDateTimeConfigEntry::setToDefault ()
+{
+    set (m_DefaultDateTime);
+}
+
+//=========================================================================
  
