@@ -4,12 +4,12 @@
 This file is part of WxWidgetsExtensions library.
 -------------------------------------------------------------------------
 
-WxWidgetsExtensions library 0.7.1
+WxExtLib (WxWidgetsExtensions) library
 -----------------------------
 
 COPYRIGHT NOTICE:
 
-WxWidgetsExtensions library Copyright (c) 2003, 2004 Daniel Kps.
+WxExtLib library Copyright (c) 2003-2007 Daniel Kps
 
 The WxWidgetsExtensions library and associated documentation files (the
 "Software") is provided "AS IS".  The author(s) disclaim all
@@ -33,11 +33,15 @@ subject to the following conditions:
  3. This Copyright notice may not be removed or altered from any 
     source or altered source distribution.
 
-End of WxWidgetsExtensions library Copyright notice
+End of WxExtLib library Copyright Notice
 
 -------------------------------------------------------------------------
 */
 
+#ifndef _NO_HEADER_INCLUDE
+
+#include "WxExtLibConfig.h"
+
 #if defined(__GNUG__) && (!defined(__APPLE__)) && (!(defined M_NoPragmaInterface))
 #   pragma implementation "MessageExtDialog.h"
 #endif
@@ -57,9 +61,19 @@ End of WxWidgetsExtensions library Copyright notice
 
 #include <wx/statline.h>
 #include <wx/image.h>
+#include <wx/artprov.h>
+
 #include "WxMisc.h"
 #include "StatusValidators.h"
 
+#endif // _NO_HEADER_INCLUDE
+
+//-------------------------------------------------------------------------
+
+double wxMessageExtDialog::s_GlobalMessageExtDialogZoom = 1.05;
+long wxMessageExtDialog::s_GlobalFlags = wxMessageExtDialog::DisplayColourBar;
+wxString wxMessageExtDialog::s_GlobalTitlePostfix;
+
 //=========================================================================
 
 BEGIN_EVENT_TABLE (wxMessageExtDialog, wxExtDialog)
@@ -76,23 +90,25 @@ wxMessageExtDialog::wxMessageExtDialog()
 }
 
 wxMessageExtDialog::wxMessageExtDialog (wxWindow *ParentWindow,
-                                    const wxString & MessageString,
-                                    const wxString & TitleString)
-  : wxExtDialog (ParentWindow, -1, "" /*TitleString*/,
-               wxDefaultPosition, wxDefaultSize,
-               wxDEFAULT_DIALOG_STYLE | wxWANTS_CHARS)
+                                        const wxString & MessageString,
+                                        const wxString & TitleString)
+  : wxExtDialog (getUsedParentWindow (ParentWindow),
+                 -1, "" /*TitleString*/,
+                 wxDefaultPosition, wxDefaultSize,
+                 wxDEFAULT_DIALOG_STYLE | wxWANTS_CHARS)
 {
     init (MessageString, TitleString);
 }
 
 void wxMessageExtDialog::Create (wxWindow *ParentWindow,
-                               const wxString & MessageString,
-                               const wxString & TitleString)
+                                 const wxString & MessageString,
+                                 const wxString & TitleString)
 {
-    wxExtDialog::Create (ParentWindow, -1, "" /* TitleString */,
-                       wxDefaultPosition, wxDefaultSize,
-                       wxDEFAULT_DIALOG_STYLE | wxWANTS_CHARS);
-                       
+    wxExtDialog::Create (getUsedParentWindow (ParentWindow),
+                         -1, "" /* TitleString */,
+                         wxDefaultPosition, wxDefaultSize,
+                         wxDEFAULT_DIALOG_STYLE | wxWANTS_CHARS);
+    
     init (MessageString, TitleString);
 }
 
@@ -118,7 +134,7 @@ void wxMessageExtDialog::init (const wxString & MessageString, const wxString &
     // wxWindow * PanelWnd = this;
     m_PanelSizer = new wxBoxSizer (wxVERTICAL);
 
-    setDialogZoom (m_GlobalMessageExtDialogZoom);
+    setDialogZoom (s_GlobalMessageExtDialogZoom);
 
     //-------------------------------------------------------------------------
     m_DialogMessageString = MessageString;
@@ -143,11 +159,17 @@ void wxMessageExtDialog::init (const wxString & MessageString, const wxString &
     // create the button font: shall be a little larger than the normal
     // dialog font:
 #if defined(__WXMSW__)
+    int ButtonFontSize = getDialogFontSize();
+
     // under MSW, the font created with the bold attribute will appear
     // higher and wider with the normal dialog font size already
     // (scope: the default dialog font, things may be different for other
     // fonts and/or true type fonts)
-    m_ButtonFont  = createDialogFont(getDialogFontSize() - 1, 
+    // so reduce (nominal) font size by one point
+    if (ButtonFontSize > 8 /* DefaultDialogFontSize */)
+      --ButtonFontSize;
+
+    m_ButtonFont  = createDialogFont(ButtonFontSize, 
                                      wxDEFAULT /* family */, 
                                      wxNORMAL /* upright/slant/italic */, 
                                      wxBOLD /* weight */,
@@ -200,14 +222,27 @@ wxMessageExtDialog & wxMessageExtDialog::addButton (const wxString & Label,
         m_ButtonFlexGridSizer -> AddGrowableCol (m_CurrentButtonIndex * 2 - 1);
       }
 
+    wxSizerAdder ButtonSizerAdder (m_ButtonFlexGridSizer);
+
     wxButton * Button;
     // WARN don't know if syntax for function pointer conversion is correct
     wxExtDialog::addButton (& Button, ButtonIdent, (wxCommandEventFunction) (& wxMessageExtDialog::OnButtonEvent), 
-                          Label, 
-                          m_ButtonFlexGridSizer);
+                            Label, 
+                            & ButtonSizerAdder);
 
     Button -> SetFont (m_ButtonFont);
-    Button -> SetBackgroundColour (wxColour (224, 224, 224));
+
+    int MajorVersionInt = 0;
+    int MinorVersionInt = 0;
+    int OsSpecifierInt = wxGetOsVersion (& MajorVersionInt, & MinorVersionInt);
+    // WinXP: do not set button's background colour because this prevents
+    // WinXP from using new WinXP button style (Win version 5.1)
+    if (!(OsSpecifierInt == wxWINDOWS_NT 
+          && ((MajorVersionInt > 5) 
+              || (MajorVersionInt == 5 && MinorVersionInt >= 1))))
+      {
+        Button -> SetBackgroundColour (wxColour (224, 224, 224));
+      }
 
 //     if (!m_LastAddedButton)
 //    Button -> SetFocus();
@@ -287,11 +322,15 @@ void wxMessageExtDialog::handleButtonEvent (wxCommandEvent& Event)
           }
       }
 
-    int ButtonIdent = Event.m_id;
+    int ButtonIdent = Event.GetId(); // Event.m_id
     if (ButtonIdent == wxID_OK)
       {
+        // WARN since wxWidgets 2.8 or so, OnOK() doesn't exist enymore, what
+        // can we do, except for disabling OnOK()?
+        // - solution is to provide our own OnOK(), OnCancel() by wxExtDialog
+
         wxExtDialog::OnOK (Event);
-        //EndModal (wxID_OK);
+        // // EndModal (wxID_OK);
       }
     else if (ButtonIdent == wxID_CANCEL)
       {
@@ -340,7 +379,7 @@ int wxMessageExtDialog::ShowModal()
     if (!m_IsControlsCreated)
       {
         createControls ();
-        m_IsControlsCreated = TRUE;
+        m_IsControlsCreated = true;
       }
 
     return wxExtDialog::ShowModal();
@@ -357,9 +396,12 @@ void wxMessageExtDialog::createControls()
     //-------------------------------------------------------------------------
     // create colour bar if required
 
-    bool IsColourBar;
-    IsColourBar = m_Style != MessageStyle;
-
+    int Style = (m_Style & StyleMask);
+    bool IsColourBar = (((s_GlobalFlags & DisplayColourBar) != 0) 
+                        && (Style != MessageStyle));
+    bool IsImage = (((s_GlobalFlags & DisplayImage) != 0)
+                    && (Style != MessageStyle));
+                    
     wxBoxSizer * GroupAllBoxSizer = new wxBoxSizer (wxHORIZONTAL);
     m_PanelSizer -> Add (GroupAllBoxSizer, 0, wxEXPAND, _B(0));
 
@@ -395,6 +437,38 @@ void wxMessageExtDialog::createControls()
     wxBoxSizer * GroupContentBoxSizer = new wxBoxSizer (wxVERTICAL);
     GroupAllBoxSizer -> Add (GroupContentBoxSizer, 0, wxLEFT|wxRIGHT|wxTOP|wxEXPAND, _B(14));
 
+    wxBoxSizer * GroupSubContentBoxSizer = new wxBoxSizer (wxHORIZONTAL);
+    GroupContentBoxSizer -> Add (GroupSubContentBoxSizer, 0, wxLEFT|wxRIGHT|wxTOP|wxEXPAND, _B(0));
+
+    //-------------------------------------------------------------------------
+    // setup image control
+
+    if (IsImage)
+      {
+        wxArtID ArtId 
+          = Style == InformationStyle ? wxART_INFORMATION
+          : Style == WarningStyle     ? wxART_WARNING
+          : Style == ErrorStyle       ? wxART_ERROR
+          : Style == CriticalErrorStyle ? wxART_ERROR
+          : Style == QuestionStyle    ? wxART_QUESTION
+          : wxEmptyString;
+
+        wxASSERT (ArtId != wxEmptyString);
+
+        if (ArtId != wxEmptyString)
+          {
+            wxBitmap Bitmap = wxArtProvider::GetBitmap (ArtId, wxART_MESSAGE_BOX, wxDefaultSize);
+            wxWindow * Window 
+              = new wxStaticBitmap (this, -1, 
+                                    Bitmap,
+                                    wxDefaultPosition, wxSize (Bitmap.GetWidth(), Bitmap.GetHeight()),
+                                    0 /* style */);
+
+            GroupSubContentBoxSizer -> Add (Window, 0, wxEXPAND, _B(0));
+            GroupSubContentBoxSizer -> Add (_B(10), _B(0));
+          }
+      }
+
     //-------------------------------------------------------------------------
     // setup text control for message
 
@@ -413,7 +487,15 @@ void wxMessageExtDialog::createControls()
     m_MessageMultiLineText.create (this, -1, 
                                    IsLongMessage ? wxBORDER_SIMPLE : wxBORDER_NONE, 
                                    TextCtrlSize,
-                                   (m_Style & IsHtmlMode) ? wxMultiLineText::wxHTML_MODE : 0);
+                                   (m_Style & IsHtmlMode) ? wxMultiLineText::wxHTML_MODE 
+#if wxCHECK_VERSION (2, 5, 0)
+                                   // HACK: use Html window always because of wxTextCtrl size
+                                   // setting problem with new wxWidgets version (2.5.2)
+                                   : wxMultiLineText::wxFORCE_HTMLWINDOW
+#else
+                                   : 0
+#endif
+                                   );
     if (m_MessageMultiLineText.getTextCtrl() != NULL)
       {
         m_MessageMultiLineText.getTextCtrl() -> SetFont (m_TextCtrlFont);
@@ -422,7 +504,7 @@ void wxMessageExtDialog::createControls()
       {
         setHtmlWindowFonts (m_MessageMultiLineText.getHtmlWindow(),
                             m_TextCtrlFont.GetPointSize(),
-                            TRUE,
+                            true,
                             FALSE);
         m_MessageMultiLineText.getHtmlWindow() -> SetBorders (5);
       }
@@ -430,7 +512,7 @@ void wxMessageExtDialog::createControls()
     m_MessageMultiLineText.setText (m_DialogMessageString);
     adjustMultiLineTextSize (m_MessageMultiLineText, 3, 10);
 
-    GroupContentBoxSizer -> Add (m_MessageMultiLineText.getWindow(), 0, wxEXPAND, _B(0));
+    GroupSubContentBoxSizer -> Add (m_MessageMultiLineText.getWindow(), 0, wxEXPAND, _B(0));
 
     //-------------------------------------------------------------------------
     GroupContentBoxSizer -> Add (new wxStaticLine (this, -1), 0, 
@@ -439,8 +521,8 @@ void wxMessageExtDialog::createControls()
     //-------------------------------------------------------------------------
     // add sizer for buttons
 
-    int AdditionalButtonSizerFlags = m_CurrentButtonIndex >= 2 
-      ? wxEXPAND : wxALIGN_CENTRE_HORIZONTAL;
+    int AdditionalButtonSizerFlags = (m_CurrentButtonIndex >= 2)
+      ? cast_enum (wxEXPAND) : cast_enum (wxALIGN_CENTRE_HORIZONTAL);
 
     GroupContentBoxSizer -> Add (m_ButtonFlexGridSizer, 0, 
                                  wxTOP
@@ -455,18 +537,18 @@ void wxMessageExtDialog::createControls()
     GroupContentBoxSizer -> Add (0, _B(14));
 
     // append postfix to title
-    if (m_GlobalTitlePostfix != wxEmptyString)
+    if (s_GlobalTitlePostfix != wxEmptyString)
       {
         if (m_TitleString != wxEmptyString)
           m_TitleString.Append (" - ");
-        m_TitleString.Append (m_GlobalTitlePostfix);
+        m_TitleString.Append (s_GlobalTitlePostfix);
       }
     SetTitle (m_TitleString);
 
     //-------------------------------------------------------------------------
     SetSizerAndFit (m_PanelSizer);
 
-    enableWatchedControlChangeNotification (TRUE);
+    enableWatchedControlChangeNotification (true);
 
     if (m_DefaultButton)
       {
@@ -482,25 +564,41 @@ void wxMessageExtDialog::setStyle (/* EStyle */ long Style)
     m_Style = Style;
 }
 
-//-------------------------------------------------------------------------
+void wxMessageExtDialog::setGlobalFlags (/* EFlags */ long Flags)
+{
+    s_GlobalFlags = Flags;
+}
 
-wxString wxMessageExtDialog::m_GlobalTitlePostfix;
+//-------------------------------------------------------------------------
 
 void  wxMessageExtDialog::setGlobalTitlePostfix (const wxString & String) // (static)
 {
-    m_GlobalTitlePostfix = String;
+    s_GlobalTitlePostfix = String;
 }
 
-double wxMessageExtDialog::m_GlobalMessageExtDialogZoom = 1.2;
-
-void wxMessageExtDialog::setGlobalMessageExtDialogZoom (double ZoomValue)
+void  wxMessageExtDialog::getGlobalTitlePostfix (wxString & String) // (static)
 {
-     m_GlobalMessageExtDialogZoom = ZoomValue;
+    String = s_GlobalTitlePostfix;
 }
 
-double wxMessageExtDialog::getGlobalMessageExtDialogZoom ()
+void wxMessageExtDialog::setGlobalMessageExtDialogZoom (double ZoomValue) // (static)
 {
-    return m_GlobalMessageExtDialogZoom;
+     s_GlobalMessageExtDialogZoom = ZoomValue;
+}
+
+double wxMessageExtDialog::getGlobalMessageExtDialogZoom () // (static)
+{
+    return s_GlobalMessageExtDialogZoom;
+}
+
+void wxMessageExtDialog::setGlobalGetDefaultParentWindowFunc (wxDefaultParentWindowFunc Func)
+{
+    wxSetGlobalDefaultParentWindowFunc (Func);
+}
+
+wxWindow * wxMessageExtDialog::getUsedParentWindow (wxWindow * ParentWindow)
+{
+    return wxGetDefaultParentWindow (ParentWindow);
 }
 
 //-------------------------------------------------------------------------
@@ -528,7 +626,7 @@ wxButton * wxMessageExtDialog::findButtonForKey (char KeyChar)
                 if (FoundButton == NULL)
                   FoundButton = (wxButton *) Window;
                 else
-                  IsDuplicatedKey = TRUE;
+                  IsDuplicatedKey = true;
               }
           }
 
@@ -597,6 +695,11 @@ int MessageExtDialog (const wxString & MessageString,
     return MessageExtDialog (NULL, MessageString, wxEmptyString, Style);
 }
 
+int MessageExtDialog (const wxString & MessageString)
+{
+    return MessageExtDialog (NULL, MessageString, wxEmptyString, wxMessageExtDialog::MessageStyle);
+}
+
 //=========================================================================
 
 extern "C" int MessageExtDialog_ (const char * MessageString,
